/*
 * $Source: /projects/splot/src/RCS/parsesup.c,v $
 * $Author: tuna $
 * $Header: /projects/splot/src/RCS/parsesup.c,v 1.4 1993/02/28 06:23:42 tuna Exp $
 * $Locker:  $
 *
 * all of this stuff used to be included at the end of plot.yacc; it
 * was moved into a separate file to facilitate debugging.
 */

#include <stdio.h>
#include <strings.h>
#include "plotio.h"
#include "defs.h"
#include "structs.h"
#include "plot.h"

/* externs for stuff declared in lex.yy.c */
extern char yytext[];

/* externs for stuff declared in y.tab.c */
extern double     currSSfreq, currSSang;
extern int        currSSfnIndex;
extern char      *curfn;
extern float      curfnstart, curfnend, curfndelta;
extern int        FUNCTION_LEN;
extern char       errorpos[];
extern pointType *ptPtr;

#define MAXSIZEOFNUM 100

char *prc       = "%f";		/* precision (?) */
char *comf      = "/tmp/_SplotComf.c";
char *inf       = "/tmp/_SplotInf";
char *outf      = "/tmp/_SplotOutf";
char *execf     = "/tmp/_SplotExecf";
char *outpoints = "/tmp/_SplotPoints";

char cmdbuf[1024];		/* used as arg to system() calls */

/* added for color support (KLJ, june 1992) */
double colorspec[3];


addPointsPair1()
{
  ptPtr = (pointType *)GetSp(sizeof(pointType));
  ptPtr->conf = NoConfInterval; /* initialized */
  if ((curves[curcur])->fstPtr == NULL)
  {
    /* means first point in curve */
    (curves[curcur])->fstPtr = ptPtr;
    ptPtr->prvPtr = NULL;
  }
  else 
  {
    ptPtr->prvPtr = (curves[curcur])->lstPtr;
    (curves[curcur])->lstPtr->nxtPtr = ptPtr;
  }
  (curves[curcur])->lstPtr = ptPtr;
  ptPtr->nxtPtr = NULL;
  (curves[curcur])->lastpoint += 1;
  /* incr point count; after 1st pt is 0 */
  ptPtr->pt[xpt]= atof(yytext);
}

addPointsPair2()
{
  ptPtr->pt[ypt] = atof(yytext);
}

addPointsConf()
{
  ptPtr->conf = atof(yytext);
}

unknowncmderror()
{
  fprintf(stderr,	
	  "***splot::Warning: unknown command near line %d\n", lineno);
  fprintf(stderr,"previous text: %s, ",errorpos);
  fprintf(stderr,"current text: %s\n",yytext);
}

newgraphproc()
{
  graph.includekey = FALSE;
  strcpy(graph.title,"");
  strcpy(graph.label,"");
  graph.font = defGraphFont;
  graph.keyfont = defGraphKeyFont;
  graph.key = kboth;
  graph.size[xpt] = defGraphXsize * 2.54;
  graph.size[ypt] = defGraphYsize * 2.54;
  setaxisdefs(xpt);
  setaxisdefs(ypt);
  currSSang = 0.0;
  currSSfnIndex = 0;
  currSSfreq = 30;
}

setaxisdefs(z)
     int z;
{
  strcpy(axis[z].label,"");
  axis[z].scale.type = linear;
  axis[z].interval = 0.0;
  axis[z].axismax = BIG;
  axis[z].axismin = BIG;
  axis[z].numberstyle = floating;
  axis[z].signif = 0;
  axis[z].unit = 1.0;
  axis[z].font = defAxisFont;
  axis[z].majorticfont = graph.font;
  axis[z].minorticfont = axis[z].majorticfont - 3;
}

newcurveproc()
{
  curcur++;			/* go to first empty slot*/
  if (curcur >= MAXCURVES)
  {
    fprintf(stderr,
	    "splot error: too many curves. Max %d\n", MAXCURVES);
    exit(1);		
  }
  curves[curcur] = (curvetype *) GetSp(sizeof(curvetype));  
  (curves[curcur])->validcurve = TRUE;
  strcpy((curves[curcur])->label,"");
  (curves[curcur])->symbol = nonesymbol;
  (curves[curcur])->symbolsize = defSymbolSize;
  (curves[curcur])->interp = linearinterp;
  (curves[curcur])->type = solid;
  (curves[curcur])->fstPtr = NULL;
  (curves[curcur])->lstPtr = NULL;
  (curves[curcur])->lastpoint = -1;
  (curves[curcur])->include = TRUE;
  (curves[curcur])->showConf = FALSE;
  (curves[curcur])->thick = defGraphThick;
  (curves[curcur])->gray = 0.0;
  (curves[curcur])->mat.grayset = FALSE;
  (curves[curcur])->mat.freq = 0.0;
  (curves[curcur])->mat.ang = 30.0;
  (curves[curcur])->mat.indx = 0;
  (curves[curcur])->mat.matset = FALSE;
  (curves[curcur])->mat.showline = TRUE;
  /* bounding curve appears by default */

  /* added the following for color support (KLJ, june 1992)
   * - indicate that user hasn't set a color yet
   * - since the color value doesn't get used unless color.set is
   *   TRUE, don't bother setting a default value for the color (and,
   *   correspondingly, make sure to set color.set to TRUE when the
   *   user provides a color value)
   */
  (curves[curcur])->color.set = FALSE;
}

handlefillproc()
{
  currSSang += 45.0; 
  /* rotate 45 degrees for each new curve */
  if (currSSang > 181.0) 
  {
    currSSang = 45.0;
    currSSfnIndex += 1;
    if (currSSfnIndex == MAXSSFNS)
    {
      currSSfnIndex = 0;
      currSSfreq += 15.0;
      if (currSSfreq > 61.0)
      {
	currSSfreq = 5.0;
      }
    }
  }
  (curves[curcur])->type = fill;
  /* bounding curve true by default */
  if ((!(curves[curcur])->mat.grayset))
    (curves[curcur])->gray = defFillGray;
  if ((!(curves[curcur])->mat.matset))
  {
    (curves[curcur])->mat.freq = currSSfreq;
    (curves[curcur])->mat.ang = currSSang;
    (curves[curcur])->mat.indx= currSSfnIndex;
  }
}

nocurverr()
{
  fprintf(stderr,
	  "splot error: no new curve command, near line %d\n", lineno);
}

notexterr()
{
  fprintf(stderr,
	  "***splot warning: error on line %d, no text entered yet!\n", lineno);
}

newtextproc()
{
  curtext++;
  if (curtext >= MAXTEXTS)
  {
    fprintf(stderr,
	    "splot error: too many text objs. Max %d\n", 
	    MAXTEXTS);
    exit(1);		
  }
  text[curtext] = (texttype *) GetSp(sizeof(texttype));
  (text[curtext])->label[0] = '\0';
  (text[curtext])->font = defTextFont;
  (text[curtext])->angle = 0;
  (text[curtext])->include = TRUE;
  (text[curtext])->validtext = TRUE;
  (text[curtext])->align = left;
}


fnerror()
{
  fprintf(stderr,
	  "***splot warning: error on line %d, incorrect function declaration!\n", 
	  lineno);
}

computfn(curfnstart,curfnend,curfndelta,curfn)
     char* curfn; 
     float curfnstart, curfnend, curfndelta;
{
  float x, y;
  FILE *fp_comf, *fp_inf, *fp_outf, *fp_outpoints, *fopen();
  char ytxt[MAXSIZEOFNUM], xtxt[MAXSIZEOFNUM];
  
  if (curcur < 0) nocurverr();
  
  /*
   * write program which computes function (in file named by comf)
   */
  fp_comf = fopen(comf,"w");
  fprintf(fp_comf,"#include <stdio.h>\n");
  fprintf(fp_comf,"#include <math.h>\n");
  fprintf(fp_comf,"main()\n{\n");
  fprintf(fp_comf,"float y,x;\n");
  fprintf(fp_comf,"while (scanf (\"%s\",&x) != EOF) {\n", prc);
  fprintf(fp_comf,"y = %s;\n",curfn);	
  fprintf(fp_comf,"printf(\"%s\\n\",y);}}\n", prc);
  fclose(fp_comf);
  
  /*
   * write input file for program (in file named by inf)
   */
  fp_inf = fopen(inf,"w");
  x = curfnstart;
  while (x <= curfnend)
  {
    fprintf(fp_inf,"%f\n",x);
    x += curfndelta;
  }
  fclose(fp_inf);
  
  /*
   * compile the program
   */
  sprintf(cmdbuf, "cc -o %s %s -lm", execf, comf);
  system(cmdbuf);

  /*
   * run the program (redirecting output into file named by outf)
   */
  sprintf(cmdbuf, "%s < %s > %s", execf, inf, outf);
  system(cmdbuf);
  
  /*
   * read the input and output files (named by inf and outf)
   * write the points for the curve (into file named by outpoints)
   */
  fp_inf       = fopen(inf,"r");
  fp_outf      = fopen(outf,"r");
  fp_outpoints = fopen(outpoints,"a");
  fprintf(fp_outpoints,"Curve Label %s\n", (curves[curcur])->label);

  while(fscanf(fp_inf, "%s", xtxt) != EOF)
  {
    strcpy(yytext, xtxt);
    fprintf(fp_outpoints,"%s, ", xtxt);
    addPointsPair1();
    
    if (fscanf(fp_outf,"%s", ytxt) != EOF) 
    {
      strcpy(yytext, ytxt);
      fprintf(fp_outpoints," %s\n", ytxt);
      addPointsPair2();
    }
    else 
    {
      fnerror();
      strcpy(yytext, "0.0");
      fprintf(fp_outpoints, " 0.0\n");
      addPointsPair2();
      break;
    }
  }
  fprintf(fp_outpoints,"\n");

  fclose(fp_inf);
  fclose(fp_outf);
  fclose(fp_outpoints);
  
  /*
   * delete the temp files
   */

  unlink(comf);
  unlink(inf);
  unlink(outf);
  unlink(execf);
  unlink(outpoints);
}
