/*
/* $Source: /am/hing/projects/splot/src/RCS/plotx11.c,v $
/* $Author: tuna $
/* $Header: /am/hing/projects/splot/src/RCS/plotx11.c,v 1.7 1992/11/03 15:37:14 tuna Exp $
/* $Locker:  $
/*/

/*
 * Copyright 1988 Anant Agarwal
 * Portions Copyright 1990, 1991 Kirk L. Johnson (KLJ)
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation.  No representations about the suitability of this software
 * for any purpose.  It is provided "as is" without express or implied 
 * warranty.
 *
 * Original Author: Anant Agarwal, MIT Laboratory for Computer Science
 *
 * Revision History:
 *  23 Jan 1990 : KLJ : changed X interface so splot windows refresh
 *  themselves instead of waiting for the user to type an "r".
 *
 *  8 Feb 1990 : KLJ : fixed gray-level color allocation code. changed
 *  font tables to use X realizations of desired fonts.
 *
 *  28 Jul 1991 : KLJ : changed redraw code to actually use clipping
 *  information from exposure events in an effort to decrease redraw
 *  time.
 *
 *  19 Oct 1991 : KLJ : rewrote color code to try and use
 *  "splot.Foreground" and "splot.Background" defaults, if present.
 *
 *  3 Nov 1992 : KLJ : added caching for fonts in order to reduce the
 *  extent to which the same font got reloaded over and over again;
 *  this caused especially disappointing performance when previewing
 *  splot graphs over a low bandwidth X connection (Xremote)
 */

#include <stdio.h>
#include <math.h>
#include <strings.h>
#include "plotio.h"

/*#include "defs.h"*/
typedef enum { FALSE, TRUE } Boolean;
typedef Boolean boolean;

#include "structs.h"
#include "plot.h"
#include "plotx11.h"
#include <X11/Xutil.h>
#include <X11/cursorfont.h>

#define NULL 0
#define COMPR 0.8
#define MULBLOW 0.5 /* size to blow up symbols */

#define CURSOR_BUSY   XC_watch
#define CURSOR_READY  XC_dot
#define CURSOR_BUTTON XC_top_left_arrow

/* X window vars */
Display *display;
GC gc;
int screen;
Window w;
Window quitb;
Window parent;
unsigned long fg_pixel;
unsigned long bg_pixel;
Cursor cursor_button;
Cursor cursor_ready;
Cursor cursor_busy;
static boolean FirstFill   = FALSE;
static Boolean FirstWindow = TRUE;

XPoint v[2];
XPoint vf[5];

/*text*/
static XFontStruct *fontstruct;

static char*     currFont = NULL;
static int       currSize = -1;
                /* eventually between MINFNTSIZ and MAXFNTSIZ points */
static LineType  currLine = none;
static double	 currThick = -2.0;
static int currThickint;
static double	 currGray = 0.0;

/* COLOR */
#define NCOLORS (14)
unsigned long colors[NCOLORS];

/*
 * this object contains the current transformed from and to points.
 * No longer do transformations in place due to the difficulty of redrawing
 */
sendx(p)
     LocalObjectType *p;
{
  LocalObjectType dummyObj, *dummyp = &dummyObj;
  double transformX(), transformY();
  int vcount = 1;
  int filled = 0;
  XFontStruct *MyGetFont();
  XPoint *GetSymbolV();
  int len = 1;
  static int CurStrWidth = 0;

  if (FirstWindow)
  {
    FirstWindow = FALSE;
    initX();
  }

  transformcoord(p, dummyp);
  /* convert to top left co-ord and return new coords in dummyp */
  switch( p->name )
  {
  case objCurLine:
  case objline :
    if( currLine != p->un.line.line || currThick != p->un.line.thick 
       || currGray != p->grey)
    {
      setConstsForCurveX(p);/* set graphics context */
    }
    if (( currLine != none )&&(currLine != fill))
    {
      vf[0].x = dummyp->from[xpt];
      vf[0].y = dummyp->from[ypt];
      vf[1].x = dummyp->un.line.to[xpt];
      vf[1].y = dummyp->un.line.to[ypt];
      /* WidthCorrect(vf,currThickint); */
      /* note that .x and .y are integers */
      XDrawLine( display, w, gc, vf[0].x,vf[0].y,vf[1].x,vf[1].y);
    }
    else if (currLine == fill)
    {
      /* draws repeated patterns of given height and width */
      /* first shade */
      vf[0].x = dummyp->from[xpt];
      vf[0].y = dummyp->from[ypt];
      vf[1].x = dummyp->un.line.to[xpt];
      vf[1].y = dummyp->un.line.to[ypt];
      vf[2].x = dummyp->un.line.to[xpt];
      vf[2].y = transformY(translate);
      vf[3].x = dummyp->from[xpt];
      vf[3].y = transformY(translate);
      vf[4].x = dummyp->from[xpt];
      vf[4].y = dummyp->from[ypt];
      /* WidthCorrect(vf,currThickint); */
      XFillPolygon(display, w, gc, vf, 5, Complex, CoordModeOrigin);
      /* then black line */
      currGray = 0.0;
      setgray(currGray);
      XDrawLine(display, w, gc, vf[0].x, vf[0].y, vf[1].x, vf[1].y);
    }
    break;
  case objsymbol :
    if( currSymbol != p->un.symbol.symbol || 
       currSize != p->un.symbol.fontsize ||
       currGray != 0)
    {
      setConstsForSymbolX(p);
    }
    if (p->un.symbol.symbol != nonesymbol)
    {
      Vsym = GetSymbolV(p, dummyp, &filled, &vcount);
      if (filled) 
	XFillPolygon(display, w, gc, Vsym, vcount, Complex, CoordModePrevious);
      else
	XDrawLines(display, w, gc, Vsym, vcount, CoordModePrevious);
    }
    break;
  case objtext :
    if( currFont != p->un.text.FF || 
       currSize != p->un.text.fontsize ||
       currGray != 0)
    {
      setConstsForTextX(p);
    }
    len = strlen(p->un.text.StrPtr);
    MyGetPt(v, p, dummyp);
    if (len) XDrawString(display, w, gc, v[0].x, v[0].y, p->un.text.StrPtr, len);
    CurStrWidth = XTextWidth(fontstruct, p->un.text.StrPtr, len);
    break;
  case objCurText :
    if( currFont != p->un.text.FF || 
       currSize != p->un.text.fontsize ||
       currGray != 0)
    {
      setConstsForTextX(p);
    }
    len = strlen(p->un.text.StrPtr);
    
    /* uses v and initial CurStrWidth from objtext */
    v[0].x += CurStrWidth;
    if (IView) v[0].y = v[0].y + p->un.text.up;/* y's not reversed in I */
    else if (XView) v[0].y = v[0].y - p->un.text.up;/*y's reversed in X */
    else fprintf(stderr,"Splot error: Only IView and XView supported\n");
    CurStrWidth = XTextWidth(fontstruct,p->un.text.StrPtr,len);
    if (len) XDrawString(display,w,gc,v[0].x,v[0].y,p->un.text.StrPtr,len);
    if (IView) v[0].y = v[0].y - p->un.text.up;/* y's not reversed in I */
    else if (XView) v[0].y = v[0].y + p->un.text.up; /*y's reversed in X */
    else fprintf(stderr,"Splot error: Only IView and XView suooprted\n");
    /*restore*/
    break;
  case objLineStartFlag :
  case objLineEndFlag :
  case objflag:
  case objflagcurves: 
  case objflagtext: 
  case objflagaxes: 
  case objflagkey: 
  case objTextStartFlag: 
  case objTextEndFlag: 
  case objdummy: 
    break;
  default:
    fprintf( stderr, "Splot.sendx: Internal Error\n" );
  }
  XFlush(display);
}

setConstsForCurveX(p)
     LocalObjectType *p;
{
  currLine = p->un.line.line;
  currThick = p->un.line.thick;
  currThickint = ceil(currThick);
  currGray = p->grey;
  setgray(currGray);
  XSetFillStyle( display, gc, FillSolid);
  
  switch( currLine )
  {
  case solid:
  case fill:
    XSetLineAttributes( display, gc, currThickint, LineSolid, CapButt, JoinMiter);
    break;
  case dashes:
    XSetLineAttributes( display, gc, currThickint, LineOnOffDash, CapButt, JoinMiter);
    XSetDashes(display,gc,0,&DashedPattern[1],DashedPattern[0]);
    break;
  case stipple:
    XSetLineAttributes( display, gc, currThickint, LineOnOffDash, CapButt, JoinMiter);
    XSetDashes(display,gc,0,&StipplePattern[1],StipplePattern[0]);
    break;
  case pat:
    XSetLineAttributes( display, gc, currThickint, LineOnOffDash, CapButt, JoinMiter);
    XSetDashes(display,gc,0,&PatPattern[1],PatPattern[0]);
    break;
  case dots:
    XSetLineAttributes( display, gc, currThickint, LineOnOffDash, CapButt, JoinMiter);
    XSetDashes(display,gc,0,&DotPattern[1],DotPattern[0]);
    break;
  default:
    XSetLineAttributes( display, gc, currThickint, LineSolid, CapButt, JoinMiter);
    break;
  }
}

setConstsForSymbolX(p)
     LocalObjectType *p;
{
  currSymbol = p->un.symbol.symbol;
  currSymbolSize = p->un.symbol.fontsize;
  currGray = 0;
  setgray(currGray);
  XSetFillStyle( display, gc, FillSolid);
  XSetLineAttributes( display, gc, 1 /*linewidth*/, LineSolid, CapButt, JoinMiter);
}

setConstsForTextX(p)
     LocalObjectType *p;
{
  XFontStruct *MyGetFont();
  
  currFont = p->un.text.FF;
  currSize = p->un.text.fontsize;
  currGray = 0;
  setgray(currGray);
  XSetFillStyle( display, gc, FillSolid);
  fontstruct = MyGetFont(currFont, currSize);	 	 
  XSetFont(display, gc, fontstruct->fid);
}

x()
{
  int    done;
  XEvent xev;
  char   keybuf[64];

  done = 0;

  while (!done)
  {
    XDefineCursor(display, w, cursor_ready);
    XDefineCursor(display, quitb, cursor_button);
    
    XNextEvent(display, &xev);

    switch (xev.type)
    {
    case Expose:
      /*
       * when we get an expose event in the main window, redraw
       */
      if (xev.xexpose.window == w)
      {
	Region     clip;
	XRectangle rect;

	XDefineCursor(display, w, cursor_busy);
	XDefineCursor(display, quitb, cursor_busy);
	FirstFill = FALSE;

	/* KLJ : 23 Jan 1990 : try to use clipping cleverly */
	clip = XCreateRegion();
	do
	{
	  rect.x      = xev.xexpose.x;
	  rect.y      = xev.xexpose.y;
	  rect.width  = xev.xexpose.width;
	  rect.height = xev.xexpose.height;
	  XUnionRectWithRegion(&rect, clip, clip);
	} while (XCheckTypedWindowEvent(display, w, Expose, &xev));
	XSetRegion(display, gc, clip);
	XDestroyRegion(clip);

	sendobj();
      }
      break;

    case KeyPress:
      /*
       * if we get a 'q' or 'Q' keypress, quit
       */
      if (XLookupString((XKeyEvent *) &xev, keybuf, 64, NULL, NULL) > 0)
	done = ((keybuf[0] == 'q') || (keybuf[0] == 'Q'));
      break;

    case ButtonRelease:
      /*
       * when we get a button release in the quit button, quit
       */
      done = (xev.xbutton.window == quitb);
      break;
      
    default:
      /*
       * ignore random events
       */
      break;
    }
  }
}

initX()
{
  int i;
  char *tmp;
  unsigned long event_mask;  
  Colormap cmap;
  XColor   xc;
  XSizeHints xsh;
  XEvent xev;

  if (IView) display = XOpenDisplay(NULL);
  else if (XView) display = XOpenDisplay(NULL);
  else fprintf(stderr,"Splot error: Only IView or XView supported\n");
  
  /* check to make sure that I have right display */
  if (display == NULL) 
  {
    fprintf(stderr,"Splot error: Cannot open display. Check environment var DISPLAY\n");
    exit(1);
  }
  screen = XDefaultScreen( display);
  gc     = XDefaultGC(display, screen);
  parent = XRootWindow( display, screen);
  cmap   = XDefaultColormap(display, screen);

  cursor_busy   = XCreateFontCursor(display, CURSOR_BUSY);
  cursor_ready  = XCreateFontCursor(display, CURSOR_READY);
  cursor_button = XCreateFontCursor(display, CURSOR_BUTTON);

  /* KLJ : 19 Oct 1991 : allow other color schemes than black on white */
  tmp = XGetDefault(display, "splot", "Foreground");
  if (tmp == NULL) 
  {
    /* no default specified by user */
    fg_pixel = BlackPixel(display, screen);
  }
  else if ((XParseColor(display, cmap, tmp, &xc) != 0) &&
	   (XAllocColor(display, cmap, &xc) != 0))
  {
    /* user specified a default, and we managed to allocate it */
    fg_pixel = xc.pixel;
  }
  else
  {
    /* user specified a default, but we couldn't allocate it */
    fprintf(stderr, "splot: unable to get foreground color '%s'", tmp);
    fprintf(stderr, " (defaulting to black)\n");
    fflush(stderr);
    fg_pixel = BlackPixel(display, screen);
  }

  tmp = XGetDefault(display, "splot", "Background");
  if (tmp == NULL) 
  {
    /* no default specified by user */
    bg_pixel = WhitePixel(display, screen);
  }
  else if ((XParseColor(display, cmap, tmp, &xc) != 0) &&
	   (XAllocColor(display, cmap, &xc) != 0))
  {
    /* user specified a default, and we managed to allocate it */
    bg_pixel = xc.pixel;
  }
  else
  {
    /* user specified a default, but we couldn't allocate it */
    fprintf(stderr, "splot: unable to get background color '%s'", tmp);
    fprintf(stderr, " (defaulting to white)\n");
    fflush(stderr);
    bg_pixel = WhitePixel(display, screen);
  }
  /* KLJ : end mods */

  XSetForeground(display, gc, fg_pixel);
  XSetBackground(display, gc, bg_pixel);

  w = XCreateSimpleWindow(display, parent,
			  0, 0, 650, 750, 2,
			  fg_pixel, bg_pixel);

  /* KLJ : 23 Jan 1990 : set window StandardProperties appropriately */
  xsh.width      = 650;
  xsh.height     = 750;
  xsh.min_width  = 650;		/* since splot doesn't deal with resizing ... */
  xsh.min_height = 750;
  xsh.max_width  = 650;
  xsh.max_height = 750;
  xsh.flags      = PSize|PMinSize|PMaxSize;
  XSetStandardProperties(display, w, "splot", "splot", None, NULL, 0, &xsh);
  /* KLJ : end mods */

  event_mask = ExposureMask|KeyPressMask;
  XSelectInput( display, w, event_mask);
  XDefineCursor(display, w, cursor_busy);
  XMapWindow( display, w );

  setup_quit_button();

  if (XGray)
  {
    /* KLJ : 19 Oct 1991 : new code for dealing with user specified colors */
    int fg_rgb[3];
    int delta_rgb[3];

    xc.pixel = fg_pixel;
    XQueryColor(display, cmap, &xc);
    fg_rgb[0] = xc.red;
    fg_rgb[1] = xc.green;
    fg_rgb[2] = xc.blue;

    xc.pixel = bg_pixel;
    XQueryColor(display, cmap, &xc);
    delta_rgb[0] = (int) xc.red   - fg_rgb[0];
    delta_rgb[1] = (int) xc.green - fg_rgb[1];
    delta_rgb[2] = (int) xc.blue  - fg_rgb[2];

    for (i=0; i<NCOLORS; i++)
    {
      xc.red   = fg_rgb[0] + delta_rgb[0] * (0.15 + 0.7 * i / (NCOLORS-1));
      xc.green = fg_rgb[1] + delta_rgb[1] * (0.15 + 0.7 * i / (NCOLORS-1));
      xc.blue  = fg_rgb[2] + delta_rgb[2] * (0.15 + 0.7 * i / (NCOLORS-1));

      if (XAllocColor(display, cmap, &xc) == 0)
      {
	fprintf(stderr, "splot : unable to allocate gray levels");
	fprintf(stderr, " (try using -g flag)\n");
	exit(1);
      }

      colors[i] = xc.pixel;
    }
  }

  /*
   * don't do any drawing until we've seen the first expose event with
   * a count field of 0
   */
  do
  {
    XNextEvent(display, &xev);
    if (xev.type != Expose) continue;
  }
  while (xev.xexpose.count != 0);
}

setup_quit_button()
{
  int button_pad;
  XFontStruct *fnt;
  int width, height;
  int top, left;
  Pixmap quitb_bg;

  button_pad = 5;		/* 5 pels of padding */

  fnt    = XLoadQueryFont(display, "variable");
  if (fnt == NULL) {
      fnt = XLoadQueryFont (display, DefFont);
      if (fnt == NULL) {
	  fprintf (stderr, "Splot: Can't find a font for the quit button\n");
	  exit (1);
      }
  }

  width  = XTextWidth(fnt, "Quit", 4) + (2 * button_pad);
  height = fnt->ascent + fnt->descent + (2 * button_pad);
  top    = 1;
  left   = 650 - (width + 3);
  
  quitb = XCreateSimpleWindow(display, w,
			      left, top, width, height, 1,
			      fg_pixel, bg_pixel);

  quitb_bg = XCreatePixmap(display, quitb, width, height,
			   DefaultDepth(display, screen));

  XSetForeground(display, gc, bg_pixel);
  XFillRectangle(display, quitb_bg, gc, 0, 0, width, height);
  XSetForeground(display, gc, fg_pixel);
  XSetFont(display, gc, fnt->fid);
  XDrawString(display, quitb_bg, gc,
	      button_pad, button_pad+fnt->ascent,
	      "Quit", 4);

  XSetWindowBackgroundPixmap(display, quitb, quitb_bg);
  XSelectInput(display, quitb, ButtonPressMask|ButtonReleaseMask);
  XDefineCursor(display, quitb, cursor_busy);
  XMapWindow(display, quitb);
}

XFontStruct *MyGetFont(localcurrFont, localcurrSize)
     int localcurrSize;/* between MINFNTSIZ and MAXFNTSIZ points */
     char *localcurrFont;
     /* takes localcurrFont and localcurrSize as arguments and returns a Font */
{
  char         *name;
  XFontStruct **cache;
  XFontStruct  *fontstruct;
  XFontStruct  *XLoadQueryFont();

  if ((localcurrSize < MINFNTSIZ) || (localcurrSize > MAXFNTSIZ))
  {
    fprintf(stderr, "Splot.GetFont: Weird font size, defaulting\n");
    localcurrSize = (MINFNTSIZ + MAXFNTSIZ)/2;
  }
  localcurrSize -= MINFNTSIZ;

  /* look into the font arrays and find (1) the name of the X font in 
   * question and (2) whether or not we already have a cached copy of
   * the XFontStruct * for said font
   */
  if (localcurrFont == Phelvetica)
  {
    name  = xhfont[localcurrSize];
    cache = &(xhfont_cache[localcurrSize]);
  }
  else if (localcurrFont == PhelveticaBold)
  {
    name  = xhbfont[localcurrSize];
    cache = &(xhbfont_cache[localcurrSize]);
  }
  else if (localcurrFont == PhelveticaItalics)
  {
    name  = xhifont[localcurrSize];
    cache = &(xhifont_cache[localcurrSize]);
  }
  else if (localcurrFont == PhelveticaBoldItalics)
  {
    name  = xhbifont[localcurrSize];
    cache = &(xhbifont_cache[localcurrSize]);
  }
  else if (localcurrFont == Ptimesroman)
  {
    name  = xtfont[localcurrSize];
    cache = &(xtfont_cache[localcurrSize]);
  }
  else if (localcurrFont == PtimesromanBold)
  {
    name  = xtbfont[localcurrSize];
    cache = &(xtbfont_cache[localcurrSize]);
  }
  else if (localcurrFont == PtimesromanItalics)
  {
    name  = xtifont[localcurrSize];
    cache = &(xtifont_cache[localcurrSize]);
  }
  else if (localcurrFont == PtimesromanBoldItalics)
  {
    name  = xtbifont[localcurrSize];
    cache = &(xtbifont_cache[localcurrSize]);
  }
  else if (localcurrFont == Greek) 
  {
    name  = GreekMap[localcurrSize];
    cache = &(GreekMap_cache[localcurrSize]);
  }
  else 
  {
    fprintf(stderr, "Splot.GetFont: Strange font name, defaulting\n");
    name  = DefFont;
    cache = &deffont_cache;
  }

  /* if we already have a cached copy of the XFontStruct * for this
   * font, reuse it; else obtain the XFontStruct *, update the cache,
   * and return it
   */
  if (*cache != NULL)
  {
    fontstruct = *cache;
  }
  else
  {
    fontstruct = XLoadQueryFont(display, name);
    if (fontstruct == NULL)
    {
      fprintf(stderr, "Splot.GetFont: unable to load font %s, defaulting\n",
	      name);

      if (deffont_cache != NULL)
      {
	fontstruct = deffont_cache;
      }
      else
      {
	fontstruct = XLoadQueryFont(display, DefFont);
	if (fontstruct == NULL)
	{
	  fprintf(stderr, "Splot.GetFont: unable to load default font (%s)\n",
		  DefFont);
	  exit(1);
	}
      }
    }

    *cache = fontstruct;
  }

  return(fontstruct);
}

MyGetPt(v , p, dummyp)
     LocalObjectType *p, *dummyp;
     XPoint v[];
{
  int len;
  int charht=0, strwt=0;
  len = strlen(p->un.text.StrPtr);
  charht = fontstruct->ascent; 
  strwt = XTextWidth(fontstruct, p->un.text.StrPtr, len);
  
  /* change tic marks on Y axis to dashes */
  if (IsItYAxisTicMark(p)) p->un.text.StrPtr = dashmark;
  
  switch( p->un.text.align )
  {
  case left:
    v[0].x = dummyp->from[xpt];
    v[0].y = dummyp->from[ypt] + (charht/2);
    break;
  case center:
    v[0].x = dummyp->from[xpt] - strwt/2;
    v[0].y = dummyp->from[ypt] + (charht/2);
    break;
  case right:
    v[0].x = dummyp->from[xpt] - strwt;
    v[0].y = dummyp->from[ypt] + (charht/2);
    break;
  }
}

IsItYAxisTicMark(p)
     LocalObjectType *p;
{
  if (
      (
       (p->un.text.angle <= -89.0)&&
       (p->un.text.angle >= -91.0)&&
       ( strncmp(ticmark, p->un.text.StrPtr, 1) == 0)
       )||
      (
       (p->un.text.angle >= 89.0)&&
       (p->un.text.angle <= 91.0)&&
       ( strncmp(ticmark, p->un.text.StrPtr, 1) == 0)
       )
      )
    return(1);
  else
    return(0);
}


transformcoord(p, dummyp)
     LocalObjectType *p, *dummyp;
{
  if (IView)
  {
    dummyp->from[ypt] = TY + (p->from[ypt]) * COMPR;
    dummyp->from[xpt] = TX + (p->from[xpt]) * COMPR;
  }
  else if (XView)
  {
    dummyp->from[ypt] = MAXY - (p->from[ypt]) * COMPR;
    dummyp->from[xpt] = MAXX + (p->from[xpt]) * COMPR;
  }
  else fprintf(stderr,"Splot error: Only IView or XView supported\n");
  
  switch( p->name )
  {
  case objCurLine:
  case objline :
    if (IView)
    {
      dummyp->un.line.to[ypt] = 
	TY + (p->un.line.to[ypt]) * COMPR;
      dummyp->un.line.to[xpt] = 
	TX + (p->un.line.to[xpt]) * COMPR;
    }
    else if (XView)
    {
      dummyp->un.line.to[ypt] = 
	MAXY - (p->un.line.to[ypt]) * COMPR;
      dummyp->un.line.to[xpt] = 
	MAXX + (p->un.line.to[xpt]) * COMPR;
    }
    else fprintf(stderr,
		 "Splot error: Only IView or XView supported\n");
    break;
  default:
    break;
  }
}

/*returns pointer to a vertex, Vsym */
XPoint *GetSymbolV(p, dummyp, Pfilled, Pvcount)
     LocalObjectType *p, *dummyp;
     int *Pfilled, *Pvcount;
{
  XPoint *VsymLoc;
  int mul, vcnt;
  switch (p->un.symbol.symbol)
  {
  case cross: *Pfilled = 0; VsymLoc = Vcross;
    break;
  case triangle: *Pfilled = 0; VsymLoc = Vtriangle;
    break;
  case trianglefilled: *Pfilled = 1; VsymLoc = Vtriangle;
    break;
  case diamond: *Pfilled = 0; VsymLoc = Vdiamond;
    break;
  case diamondfilled: *Pfilled = 1; VsymLoc = Vdiamond;
    break;
  case square: *Pfilled = 0; VsymLoc = Vsquare;
    break;
  case squarefilled: *Pfilled = 1; VsymLoc = Vsquare;
    break;
  case rectangle: *Pfilled = 0; VsymLoc = Vrectangle;
    break;
  case rectanglefilled: *Pfilled = 1; VsymLoc = Vrectangle;
    break;
  case circle: *Pfilled = 0; VsymLoc = Vcircle;
    break;
  case circlefilled: *Pfilled = 1; VsymLoc = Vcircle;
    break;
  case bullet: *Pfilled = 1; VsymLoc = Vcircle;
    break;
  case star: *Pfilled = 0; VsymLoc = Vstar;
    break;
  case plus: *Pfilled = 0; VsymLoc = Vplus;
    break;
  default:
    break;
  }
  
  if (p->un.symbol.symbol == circle) mul = p->un.symbol.fontsize/1.5;
  else mul = p->un.symbol.fontsize;
  mul = mul * MULBLOW;
  
  /* The segment count plus first point is in the first element.x */
  *Pvcount = VsymLoc[0].x;
  
  /* The second element pos must be added to origin */
  vcnt = 0;
  VsymTmp[vcnt].x = dummyp->from[xpt] + VsymLoc[vcnt+1].x * mul;
  VsymTmp[vcnt].y = dummyp->from[ypt] + VsymLoc[vcnt+1].y * mul;
  
  /* all other points are relative */
  vcnt++;
  while (vcnt < *Pvcount)
  {
    VsymTmp[vcnt].x = VsymLoc[vcnt+1].x * mul;
    VsymTmp[vcnt].y = VsymLoc[vcnt+1].y * mul;
    vcnt++;
  }
  return(VsymTmp);
}

/* corrects line length based on line width for good abutting of lines */
WidthCorrect(v,width)
     XPoint v[];
     int width;
{
  if (v[0].x > v[1].x) v[0].x -= width;
  else if (v[1].x > v[0].x) v[1].x -= width;
  else
  {
    v[0].x -= width / 2; 
    v[1].x -= width / 2; 
  }
  
  if (v[0].y > v[1].y) v[0].y -= width;
  else if (v[1].y > v[0].y) v[1].y -= width;
  else
  {
    v[0].y -= width / 2; 
    v[1].y -= width / 2; 
  }
}

WhiteOutX()
{
  double transformX(), transformY();
  
  vf[0].x = transformX(translate);
  vf[0].y = transformY(translate);
  
  vf[1].x = transformX(translate + graphsize[xpt] * 72.0 / 2.54);
  vf[1].y = transformY(translate);
  
  vf[2].x = transformX(translate + graphsize[xpt] * 72.0 / 2.54);
  vf[2].y = transformY(translate + graphsize[ypt] * 72.0 / 2.54);
  
  vf[3].x = transformX(translate);
  vf[3].y = transformY(translate + graphsize[ypt] * 72.0 / 2.54);
  
  vf[4].x = transformX(translate);
  vf[4].y = transformY(translate);
  
  XFillPolygon(display,w,gc,vf,5,Complex,CoordModeOrigin);
  FirstFill = TRUE;
}

double transformY(y)
     double y;
{
  if (IView) return(y * COMPR);
  else if (XView) return(MAXY - y * COMPR);
  else fprintf(stderr,"Splot error: Only IView or XView supported\n");
}


double transformX(x)
     double x;
{
  if (IView) return(x * COMPR);
  else if (XView) return(MAXX + x * COMPR);
  else fprintf(stderr,"Splot error: Only IView or XView supported\n");
}

/* fixes gray level pixel */
setgray(currGray)
     double currGray;
{
  int pixel;
  int index;
  
  if (XGray == 0)
  {
    if (currGray <= 0.9)
      pixel = fg_pixel;
    else
      pixel = bg_pixel;
  }
  else if (currGray <= 0.01)
  {
    pixel = fg_pixel;
  }
  else if (currGray >= 0.99)
  {
    pixel = bg_pixel;
  }
  else
  {
    index = currGray * NCOLORS;
    if (index >= NCOLORS)
      index = NCOLORS - 1;

    pixel = colors[index];
  }
  
  XSetForeground(display, gc, pixel);
}


/*
 * complain appropriately about attempts to plot IViews stuff
 */

i()
{
  fprintf(stderr,
	  "Error: Cannot display IViews graph on this machine\n");
  fprintf(stderr,"Use splot with the -i switch, instead.\n");
  exit(1);
}

sendi( p )
     LocalObjectType *p;
{
  fprintf(stderr,
	  "Error: Cannot display IViews graph on this machine\n");
  fprintf(stderr,"Use isplot with the -i switch, instead.\n");
  exit(1);
}


/****debug****/
debug(msg)
     char *msg;
{
  fprintf(stderr, "---- %s\n", msg);
  fflush(stderr);
}
